@@ -9,8 +9,9 @@ import internetarchive
 
 from internetarchive.config import parse_config_file
 from datetime import datetime
-from youtube_dlc import YoutubeDL
-from .utils import (check_is_file_empty, EMPTY_ANNOTATION_FILE)
+from yt_dlp import YoutubeDL
+from .utils import (get_itemname, check_is_file_empty,
+                    EMPTY_ANNOTATION_FILE)
 from logging import getLogger
 from urllib.parse import urlparse
 
@@ -38,6 +39,8 @@ class TubeUp(object):
                                '~/.tubeup'.
         :param ia_config_path:  Path to an internetarchive config file, will
                                 be used in uploading the file.
+        :param output_template: A template string that will be used to
+                                generate the output filenames.
         """
         self.dir_path = dir_path
         self.verbose = verbose
@@ -79,13 +82,17 @@ class TubeUp(object):
                                       DOWNLOAD_DIR_NAME)
         }
 
-    def get_resource_basenames(self, urls, proxy_url=None, ydl_username=None,
-                               ydl_password=None, use_download_archive=False):
+    def get_resource_basenames(self, urls,
+                               cookie_file=None, proxy_url=None,
+                               ydl_username=None, ydl_password=None,
+                               use_download_archive=False,
+                               ignore_existing_item=False):
         """
         Get resource basenames from an url.
 
         :param urls:                  A list of urls that will be downloaded with
                                       youtubedl.
+        :param cookie_file:           A cookie file for YoutubeDL.
         :param proxy_url:             A proxy url for YoutubeDL.
         :param ydl_username:          Username that will be used to download the
                                       resources with youtube_dl.
@@ -95,10 +102,33 @@ class TubeUp(object):
                                       This will download only videos not listed in
                                       the archive file. Record the IDs of all
                                       downloaded videos in it.
+        :param ignore_existing_item:  Ignores the check for existing items on archive.org.
         :return:                      Set of videos basename that has been downloaded.
         """
         downloaded_files_basename = set()
 
+        def check_if_ia_item_exists(infodict):
+            itemname = get_itemname(infodict)
+            item = internetarchive.get_item(itemname)
+            if item.exists and self.verbose:
+                print("\n:: Item already exists. Not downloading.")
+                print('Title: %s' % infodict['title'])
+                print('Video URL: %s\n' % infodict['webpage_url'])
+                return True
+            return False
+
+        def ydl_progress_each(entry):
+            if not entry:
+                self.logger.warning('Video "%s" is not available. Skipping.' % url)
+                return
+            if ydl.in_download_archive(entry):
+                return
+            if not check_if_ia_item_exists(entry):
+                ydl.extract_info(entry['webpage_url'])
+                downloaded_files_basename.update(self.create_basenames_from_ydl_info_dict(ydl, entry))
+            else:
+                ydl.record_download_archive(entry)
+
         def ydl_progress_hook(d):
             if d['status'] == 'downloading' and self.verbose:
                 if d.get('_total_bytes_str') is not None:
@@ -124,12 +154,11 @@ class TubeUp(object):
                 sys.stdout.flush()
 
             if d['status'] == 'finished':
-                msg = 'Downloaded %s' % d['filename']
+                msg = '\nDownloaded %s' % d['filename']
 
                 self.logger.debug(d)
                 self.logger.info(msg)
                 if self.verbose:
-                    print('\n%s' % d)
                     print(msg)
 
             if d['status'] == 'error':
@@ -140,19 +169,25 @@ class TubeUp(object):
                 if self.verbose:
                     print(msg)
 
-        ydl_opts = self.generate_ydl_options(ydl_progress_hook, proxy_url,
+        ydl_opts = self.generate_ydl_options(ydl_progress_hook,
+                                             cookie_file, proxy_url,
                                              ydl_username, ydl_password,
                                              use_download_archive)
 
         with YoutubeDL(ydl_opts) as ydl:
             for url in urls:
-                # Get the info dict of the url, it also download the resources
-                # if necessary.
-                info_dict = ydl.extract_info(url)
+                if not ignore_existing_item:
+                    # Get the info dict of the url
+                    info_dict = ydl.extract_info(url, download=False)
 
-                downloaded_files_basename.update(
-                    self.create_basenames_from_ydl_info_dict(ydl, info_dict)
-                )
+                    if info_dict.get('_type', 'video') == 'playlist':
+                        for entry in info_dict['entries']:
+                            ydl_progress_each(entry)
+                    else:
+                        ydl_progress_each(info_dict)
+                else:
+                    info_dict = ydl.extract_info(url)
+                    downloaded_files_basename.update(self.create_basenames_from_ydl_info_dict(ydl, info_dict))
 
         self.logger.debug(
             'Basenames obtained from url (%s): %s'
@@ -194,6 +229,7 @@ class TubeUp(object):
 
     def generate_ydl_options(self,
                              ydl_progress_hook,
+                             cookie_file=None,
                              proxy_url=None,
                              ydl_username=None,
                              ydl_password=None,
@@ -201,7 +237,7 @@ class TubeUp(object):
                              ydl_output_template=None):
         """
         Generate a dictionary that contains options that will be used
-        by youtube_dl.
+        by yt-dlp.
 
         :param ydl_progress_hook:     A function that will be called during the
                                       download process by youtube_dl.
@@ -229,7 +265,7 @@ class TubeUp(object):
             'continuedl': True,
             'retries': 9001,
             'fragment_retries': 9001,
-            'forcejson': True,
+            'forcejson': False,
             'writeinfojson': True,
             'writedescription': True,
             'writethumbnail': True,
@@ -255,6 +291,9 @@ class TubeUp(object):
             'progress_hooks': [ydl_progress_hook]
         }
 
+        if cookie_file is not None:
+            ydl_opts['cookiefile'] = cookie_file
+
         if proxy_url is not None:
             ydl_opts['proxy'] = proxy_url
 
@@ -278,18 +317,20 @@ class TubeUp(object):
         :param custom_meta:    A custom meta, will be used by internetarchive
                                library when uploading to archive.org.
         :return:               A tuple containing item name and metadata used
-                               when uploading to archive.org.
+                               when uploading to archive.org and whether the item
+                               already exists.
         """
         json_metadata_filepath = videobasename + '.info.json'
-        with open(json_metadata_filepath) as f:
+        with open(json_metadata_filepath, 'r', encoding='utf-8') as f:
             vid_meta = json.load(f)
 
-        itemname = ('%s-%s' % (vid_meta['extractor'],
-                               vid_meta['display_id']))
-
-        # Replace illegal characters within identifer
-        itemname = re.sub(r'\W+', '-', itemname)
+        # Exit if video download did not complete, don't upload .part files to IA
+        for ext in ['*.part', '*.f303.*', '*.f302.*', '*.ytdl', '*.f251.*', '*.248.*', '*.f247.*', '*.temp']:
+            if glob.glob(videobasename + ext):
+                msg = 'Video download incomplete, please re-run or delete video stubs in downloads folder, exiting...'
+                raise Exception(msg)
 
+        itemname = get_itemname(vid_meta)
         metadata = self.create_archive_org_metadata_from_youtubedl_meta(
             vid_meta)
 
@@ -320,7 +361,7 @@ class TubeUp(object):
             metadata.update(custom_meta)
 
         # Parse internetarchive configuration file.
-        parsed_ia_s3_config = parse_config_file(self.ia_config_path)[1]['s3']
+        parsed_ia_s3_config = parse_config_file(self.ia_config_path)[2]['s3']
         s3_access_key = parsed_ia_s3_config['access']
         s3_secret_key = parsed_ia_s3_config['secret']
 
@@ -340,9 +381,11 @@ class TubeUp(object):
 
         return itemname, metadata
 
-    def archive_urls(self, urls, custom_meta=None, proxy=None,
+    def archive_urls(self, urls, custom_meta=None,
+                     cookie_file=None, proxy=None,
                      ydl_username=None, ydl_password=None,
-                     use_download_archive=False):
+                     use_download_archive=False,
+                     ignore_existing_item=False):
         """
         Download and upload videos from youtube_dl supported sites to
         archive.org
@@ -351,6 +394,7 @@ class TubeUp(object):
                                       to archive.org
         :param custom_meta:           A custom metadata that will be used when
                                       uploading the file with archive.org.
+        :param cookie_file:           A cookie file for YoutubeDL.
         :param proxy_url:             A proxy url for YoutubeDL.
         :param ydl_username:          Username that will be used to download the
                                       resources with youtube_dl.
@@ -360,12 +404,13 @@ class TubeUp(object):
                                       This will download only videos not listed in
                                       the archive file. Record the IDs of all
                                       downloaded videos in it.
+        :param ignore_existing_item:  Ignores the check for existing items on archive.org.
         :return:                      Tuple containing identifier and metadata of the
                                       file that has been uploaded to archive.org.
         """
         downloaded_file_basenames = self.get_resource_basenames(
-            urls, proxy, ydl_username, ydl_password, use_download_archive)
-
+            urls, cookie_file, proxy, ydl_username, ydl_password, use_download_archive,
+            ignore_existing_item)
         for basename in downloaded_file_basenames:
             identifier, meta = self.upload_ia(basename, custom_meta)
             yield identifier, meta
@@ -391,23 +436,19 @@ class TubeUp(object):
         :return:
         """
         licenseurl = ''
+        licenses = {
+            "Creative Commons Attribution license (reuse allowed)": "https://creativecommons.org/licenses/by/3.0/",
+            "Attribution-NonCommercial-ShareAlike": "https://creativecommons.org/licenses/by-nc-sa/2.0/",
+            "Attribution-NonCommercial": "https://creativecommons.org/licenses/by-nc/2.0/",
+            "Attribution-NonCommercial-NoDerivs": "https://creativecommons.org/licenses/by-nc-nd/2.0/",
+            "Attribution": "https://creativecommons.org/licenses/by/2.0/",
+            "Attribution-ShareAlike": "https://creativecommons.org/licenses/by-sa/2.0/",
+            "Attribution-NoDerivs": "https://creativecommons.org/licenses/by-nd/2.0/"
+        }
+
         if 'license' in vid_meta and vid_meta['license']:
-            if vid_meta['extractor'].lower() == 'youtube':
-                if vid_meta['license'] == 'Creative Commons Attribution license (reuse allowed)':
-                    licenseurl = 'https://creativecommons.org/licenses/by/3.0/'
-            elif vid_meta['extractor'].lower() == 'flickr':
-                if vid_meta['license'] == 'Attribution-NonCommercial-ShareAlike':
-                    licenseurl = 'https://creativecommons.org/licenses/by-nc-sa/2.0/'
-                elif vid_meta['license'] == 'Attribution-NonCommercial':
-                    licenseurl = 'https://creativecommons.org/licenses/by-nc/2.0/'
-                elif vid_meta['license'] == 'Attribution-NonCommercial-NoDerivs':
-                    licenseurl = 'https://creativecommons.org/licenses/by-nc-nd/2.0/'
-                elif vid_meta['license'] == 'Attribution':
-                    licenseurl = 'https://creativecommons.org/licenses/by/2.0/'
-                elif vid_meta['license'] == 'Attribution-ShareAlike':
-                    licenseurl = 'https://creativecommons.org/licenses/by-sa/2.0/'
-                elif vid_meta['license'] == 'Attribution-NoDerivs':
-                    licenseurl = 'https://creativecommons.org/licenses/by-nd/2.0/'
+            licenseurl = licenses.get(vid_meta['license'])
+
         return licenseurl
 
     @staticmethod
@@ -427,7 +468,9 @@ class TubeUp(object):
         # Some video services don't tell you the uploader,
         # use our program's name in that case.
         try:
-            if 'uploader' in vid_meta and vid_meta['uploader']:
+            if vid_meta['extractor_key'] == 'TwitchClips' and 'creator' in vid_meta and vid_meta['creator']:
+                uploader = vid_meta['creator']
+            elif 'uploader' in vid_meta and vid_meta['uploader']:
                 uploader = vid_meta['uploader']
             elif 'uploader_url' in vid_meta and vid_meta['uploader_url']:
                 uploader = vid_meta['uploader_url']
@@ -436,8 +479,6 @@ class TubeUp(object):
         except TypeError:  # apparently uploader is null as well
             uploader = 'tubeup.py'
 
-        uploader_url = vid_meta.get('uploader_url', videourl)
-
         try:  # some videos don't give an upload date
             d = datetime.strptime(vid_meta['upload_date'], '%Y%m%d')
             upload_date = d.isoformat().split('T')[0]
@@ -456,10 +497,10 @@ class TubeUp(object):
             try:
                 for category in vid_meta['categories']:
                     tags_string += '%s;' % category
-            except:
-                print ("No categories found.")
+            except Exception:
+                print("No categories found.")
 
-        if 'tags' in vid_meta: # some video services don't have tags
+        if 'tags' in vid_meta:  # some video services don't have tags
             try:
                 if 'tags' in vid_meta is None:
                     tags_string += '%s;' % vid_meta['id']
@@ -467,18 +508,24 @@ class TubeUp(object):
                 else:
                     for tag in vid_meta['tags']:
                         tags_string += '%s;' % tag
-            except:
-                print ("Unable to process tags successfully.")
+            except Exception:
+                print("Unable to process tags successfully.")
+
+        # IA's subject field has a 255 bytes length limit, so we need to truncate tags_string
+        while len(tags_string.encode('utf-8')) > 255:
+            tags_list = tags_string.split(';')
+            tags_list.pop()
+            tags_string = ';'.join(tags_list)
 
         # license
         licenseurl = TubeUp.determine_licenseurl(vid_meta)
 
         # if there is no description don't upload the empty .description file
         description_text = vid_meta.get('description', '')
-
-        description = ('{0} <br/><br/>Source: <a href="{1}">{2}</a>'
-                       '<br/>Uploader: <a href="{3}">{4}</a>').format(
-            description_text, videourl, videourl, uploader_url, uploader)
+        if description_text is None:
+            description_text = ''
+        # archive.org does not display raw newlines
+        description = re.sub('\r?\n', '<br>', description_text)
 
         metadata = dict(
             mediatype=('audio' if collection == 'opensource_audio'
@@ -497,4 +544,10 @@ class TubeUp(object):
             # powered uploads, per request from archive.org
             scanner='TubeUp Video Stream Mirroring Application {}'.format(__version__))
 
+        # add channel url if it exists
+        if 'uploader_url' in vid_meta:
+            metadata["channel"] = vid_meta["uploader_url"]
+        elif 'channel_url' in vid_meta:
+            metadata["channel"] = vid_meta["channel_url"]
+
         return metadata
